from .anime import ConfigAnime
from .backup import ConfigBackupRestore
from .general import ConfigGeneral
from .index import Config
from .notifications import ConfigNotifications
from .post_processing import ConfigPostProcessing
from .providers import ConfigProviders
from .search import ConfigSearch
from .shares import ConfigShares
from .subtitles import ConfigSubtitles
