/* to make room for the icon, a width needs to be set here
 * @note Copied from jquery-ui@1.12.1
 */
.ui-button-icon-only {
	width: 2em;
	box-sizing: border-box;
	text-indent: -9999px;
	white-space: nowrap;
}

/**
 * Modify the scrollable element in jquery ui-dialog
 */
.ui-dialog.ui-dialog-scrollable-by-child .ui-dialog-content {
  overflow: hidden;
  padding-right: 0;
}

.ui-dialog.ui-dialog-scrollable-by-child .ui-dialog-content .ui-dialog-scrollable-child {
  overflow: auto;
  padding-right: 0.5em;
}
